@@ -76,6 +76,9 @@ class Generator:
     # Whether or not null ordering is supported in order by
     NULL_ORDERING_SUPPORTED = True
 
+    # Whether or not locking reads (i.e. SELECT ... FOR UPDATE/SHARE) are supported
+    LOCKING_READS_SUPPORTED = False
+
     # Always do union distinct or union all
     EXPLICIT_UNION = False
 
@@ -293,10 +296,10 @@ class Generator:
         )
         return f"({self.sep('')}{this_sql}{self.seg(')', sep='')}"
 
-    def no_identify(self, func: t.Callable[[], str]) -> str:
+    def no_identify(self, func: t.Callable[..., str], *args, **kwargs) -> str:
         original = self.identify
         self.identify = False
-        result = func()
+        result = func(*args, **kwargs)
         self.identify = original
         return result
 
@@ -1078,7 +1081,7 @@ class Generator:
     def lambda_sql(self, expression: exp.Lambda, arrow_sep: str = "->") -> str:
         args = self.expressions(expression, flat=True)
         args = f"({args})" if len(args.split(",")) > 1 else args
-        return self.no_identify(lambda: f"{args} {arrow_sep} {self.sql(expression, 'this')}")
+        return f"{args} {arrow_sep} {self.sql(expression, 'this')}"
 
     def lateral_sql(self, expression: exp.Lateral) -> str:
         this = self.sql(expression, "this")
@@ -1106,6 +1109,14 @@ class Generator:
         this = self.sql(expression, "this")
         return f"{this}{self.seg('OFFSET')} {self.sql(expression, 'expression')}"
 
+    def lock_sql(self, expression: exp.Lock) -> str:
+        if self.LOCKING_READS_SUPPORTED:
+            lock_type = "UPDATE" if expression.args["update"] else "SHARE"
+            return self.seg(f"FOR {lock_type}")
+
+        self.unsupported("Locking reads using 'FOR UPDATE/SHARE' are not supported")
+        return ""
+
     def literal_sql(self, expression: exp.Literal) -> str:
         text = expression.this or ""
         if expression.is_string:
@@ -1226,6 +1237,7 @@ class Generator:
             self.sql(expression, "order"),
             self.sql(expression, "limit"),
             self.sql(expression, "offset"),
+            self.sql(expression, "lock"),
             sep="",
         )
 
@@ -1474,9 +1486,16 @@ class Generator:
         return f"(SELECT {self.sql(unnest)})"
 
     def interval_sql(self, expression: exp.Interval) -> str:
-        this = self.sql(expression, "this")
-        this = f" {this}" if this else ""
-        unit = self.sql(expression, "unit")
+        this = expression.args.get("this")
+        if this:
+            this = (
+                f" {this}"
+                if isinstance(this, exp.Literal) or isinstance(this, exp.Paren)
+                else f" ({this})"
+            )
+        else:
+            this = ""
+        unit = expression.args.get("unit")
         unit = f" {unit}" if unit else ""
         return f"INTERVAL{this}{unit}"
 
@@ -1836,7 +1855,7 @@ class Generator:
 
     def userdefinedfunction_sql(self, expression: exp.UserDefinedFunction) -> str:
         this = self.sql(expression, "this")
-        expressions = self.no_identify(lambda: self.expressions(expression))
+        expressions = self.no_identify(self.expressions, expression)
         expressions = (
             self.wrap(expressions) if expression.args.get("wrapped") else f" {expressions}"
         )
